"""objects.

This submodule includes the two-dimensional object classes.

"""

from .object import Object
from .system import SystemObjects
from .single import SingleObject

__all__ = ['Object', 'SystemObjects', 'SingleObject']
